
********
********  This code creates the Figure A.3 in Appendix A.9
********

clear
timer clear
clear matrix
clear mata
set more off
set matsize 6000
graph set window fontface "Garamond"

** <<IMPORTANT>> Note: Use directory of this code as working directory


*******
******* PART 1: Create databases for histograms
*******

foreach i of numlist 200 {

	forval j = 4(1)6 {
		
		clear all
		
		*** Create databases
		local outfilename = "data/hazards_young_old_reg_"+"`j'"+"_"+"`i'"+"_nofracp_double_by_groupid_april2020"
		save "`outfilename'", emptyok replace

		foreach MKT in ATL BOS CHI DAL HOU LA NYC PHI SF {

			foreach CAT in beer coldcer fzdinent fzpizza saltsnck yogurt{
				***
				*** Load data
				***
				local infilename = "I:/data/IRI/dispersion/allprices_allyears_wsales_"+"`CAT'"+"_"+"`MKT'"
				use "`infilename'"
				
				gen CAT = "`CAT'"
				gen MKT = "`MKT'"
				display "`MKT'"
				display "`CAT'"
				
				*** Define relevant price variable
				gen P = REGPRICE
	
				*** KEEP ONLY NEEDED DATA
				keep CAT MKT WEEK group_id UNITS P ITEM DOLLARS IRI_KEY
				drop if P==.

				xtset group_id WEEK
				
				
				***
				*** Keep only observations for which the price is to the cent (STRICT DEFINITION)
				***
				gen p_dec_100_cent_round = round((P-floor(P))*100-round((P-floor(P))*100),0.001)
				drop if p_dec_100_cent_round~=0
				drop p_*
				
				** Don't allow holes (a missing week means starting a new spell)
				by group_id: gen last_price = L.P
				by group_id: gen dweek = WEEK-WEEK[_n-1]
				gen anyprice_ok = 1 if P!=last_price & P!=. & dweek==1
				replace anyprice_ok = 0 if P==last_price & P!=.		

				*** Define censored spell (first spell)
				by group_id: gen cumsum = sum(anyprice_ok)
				by group_id: replace cumsum = L.cumsum if cumsum==1&L.cumsum==0
				replace anyprice_ok=. if cumsum==0

				*** Define spell length (how long since last new price)
				gen last_change_any=WEEK if anyprice_ok==1
				replace last_change_any=L.last_change_any if anyprice_ok==0&last_change_any==.
				gen spell_any = WEEK-last_change_any+1
				gen lag_spell_any = L.spell_any

				*** Price change dummy
				gen dp = ln(P/L.P)
				gen dpdum = 0 if dp!=.
				replace dpdum = 1 if dp!=.&dp!=0

				* Define market size
				egen sum_DOLLARS = total(DOLLARS)

				
				******************
				***** ONLY KEEP SPELLS LE 26 weeks
				******************
				drop if lag_spell_any>26
				
				**
				** dp freq for young and old spells (by group_id)
				**
				
				* Minimum # of observations for group_id
				bysort group_id: egen nn=count(anyprice_ok)
				bysort group_id: egen nn_young=count(anyprice_ok) if lag_spell_any<`j'
				bysort group_id: egen nn_old=count(anyprice_ok) if lag_spell_any>=`j'
				
				* dp freq for young and old spells
				bysort group_id: egen dp_young=mean(anyprice_ok) if lag_spell_any<`j'
				bysort group_id: egen dp_old=mean(anyprice_ok) if lag_spell_any>=`j'
				
				* collapse data
				collapse (mean)  dp_young  dp_old nn nn_young  nn_old anyprice_ok (count) nb_weeks=WEEK nb_anyprice=anyprice_ok (sum) DOLLARS (first) sum_DOLLARS, by(CAT MKT IRI_KEY ITEM group_id)
				
				* Minimum # of observations for group_id
				drop if nn<`i'|nn_young<`j'*0.2|nn_old<`j'*0.2|nn_young==.|nn_old==.
			

				*** SAVE DATA (at ITEM level)
				append using "`outfilename'"
				save  "`outfilename'", replace

				}		

		}

	}
}




******
****** PART 2: HISTOGRAMS FOR FIGURE A.3
******



*******  HISTOGRAMS - ACTUAL DATA
clear all

foreach i of numlist 200 {

	forval j = 4(1)6 {
		
		clear all
	
		*** Create databases
		local infilename = "data/hazards_young_old_reg_"+"`j'"+"_"+"`i'"+"_nofracp_double_by_groupid_april2020"
		*"I:/data/IRI/fear/hazards_young_old_reg_"+"`j'"+"_"+"`i'"+"_nofracp_double_by_groupid_9jan2019"
		use "`infilename'"
		
		gen nobs_min=`i'
		gen threshold=`j'
		
		gen diff_young_old = dp_old-dp_young
		hist diff_young_old if diff_young_old>-0.4&diff_young_old<0.2, xtitle("Hazard slope") title("{&Gamma} = `j' weeks") graphregion(fcolor(white) ilcolor(white) lcolor(white)) saving(graph_`j'_`i', replace)
		
		}
}



*******  HISTOGRAMS - SIMULATED DATA ********
clear all

foreach i of numlist 200 {

	forval j = 4(1)6 {
		
		clear all
	
		* Note : this simulated database was created by a Matlab code described in the model replication material
		import excel "data\hazard_cells_n200.xlsx", firstrow
		
		gen nobs_min=`i'
		gen threshold=`j'
		
		gen diff_young_old = old_`j'_`i'-young_`j'_`i'
		hist diff_young_old if diff_young_old>-0.4&diff_young_old<0.2, xtitle("Hazard slope")  xscale(range(-0.4 0.2)) xlabel(-0.4(0.2)0.2) graphregion(fcolor(white) ilcolor(white) lcolor(white)) title("{&Gamma} = `j' weeks") saving(simul_graph_`j'_`i', replace)
		
			}
}


*************************
*** Data vs model (Figure A.3.)
*************************
graph combine graph_4_200.gph graph_5_200.gph graph_6_200.gph, graphregion(fcolor(white) ilcolor(white) lcolor(white)) rows(3) cols(1) title("Data", size(large)) saving(data_column, replace)
graph combine simul_graph_4_200.gph simul_graph_5_200.gph simul_graph_6_200.gph, graphregion(fcolor(white) ilcolor(white) lcolor(white)) rows(3) cols(1) title("Simulations", size(large)) saving(sim_column, replace)
graph combine data_column.gph sim_column.gph, graphregion(fcolor(white) ilcolor(white) lcolor(white)) rows(1) cols(2)
collapse (mean) diff_young_old significant positive significant_positive [aw=DOLLARS]


