********
********  This code is for Table III, panel C: Avg hazard slope
********

clear
timer clear
clear matrix
clear mata
set more off
set matsize 6000
set type double

*** Create databases
save "data/dbase_for_hazard_slope_april2020", emptyok replace


foreach MKT in ATL BOS CHI DAL HOU LA NYC PHI SF {

	foreach CAT in beer coldcer fzdinent fzpizza saltsnck yogurt{

		***
		*** Load data
		***
		local infilename ="data/allprices_allyears_wsales_"+"`CAT'"+"_"+"`MKT'"
		use "`infilename'"
		compress
		
		gen CAT = "`CAT'"
		gen MKT = "`MKT'"
		display "`MKT'"
		display "`CAT'"
		
		*** Define relevant price variable
		gen P = REGPRICE

		*** KEEP ONLY NEEDED DATA
		keep CAT MKT WEEK group_id UNITS P ITEM DOLLARS IRI_KEY
		drop if P==.

		***
		*** Keep only observations for which the price is to the cent (STRICT DEFINITION)
		***
		gen p_dec_100_cent_round = round((P-floor(P))*100-round((P-floor(P))*100),0.001)
		drop if p_dec_100_cent_round~=0
		drop p_*
		
		xtset group_id WEEK

		
		***
		*** DEAL WITH GAPS IN QUOTE LINES
		***

		* Maximum accepted length of a gap for it to be filled
		local maxgap=1

				
		** Don't allow holes (a missing week means starting a new spell)
		by group_id: gen last_price = L.P
		by group_id: gen dweek = WEEK-WEEK[_n-1]
		gen anyprice_ok = 1 if P!=last_price & P!=. & dweek==1
		replace anyprice_ok = 0 if P==last_price & P!=.	
		
		
		* Get rid of unwanted observations and variables
		drop if P==.
		drop dweek*

		
		***
		*** DEFINE SPELLS
		***
		xtset group_id WEEK
		by group_id: gen dweek = WEEK-WEEK[_n-1]
		
		* Define spell start and spell end indicators (allow for gaps up to a certain length)
		gen spell_start = 1 if anyprice_ok==1|dweek==.|dweek>`maxgap'
		by group_id: gen spell_end = 1 if anyprice_ok[_n+1]==1|dweek[_n+1]==.|dweek[_n+1]>`maxgap'
		
		* Define spell number (to identify them)
		gen spell_nb = 1
		gen temp1 = 0
		replace temp1 = 1 if spell_start==.
		gen temp2 = 0
		replace temp2 = 1 if spell_start==1
		replace spell_nb = spell_nb[_n-1]*temp1 + (spell_nb[_n-1]+1)*temp2 if spell_nb[_n-1]!=.

		* Identify left-censored spells
		drop temp*
		sort spell_nb WEEK
		by spell_nb: gen temp1 = 1 if dweek==.|dweek>`maxgap'
		by spell_nb: egen left_censored = min(temp1)
		drop temp*
		
		* Define first and last spells for each quote line
		bysort group_id: egen temp_min=min(WEEK)
		bysort group_id: egen temp_max=max(WEEK)
		gen temp1 = 1 if WEEK==temp_min
		gen temp2 = 1 if WEEK==temp_max
		bysort spell_nb: egen first_spell = min(temp1)
		bysort spell_nb: egen last_spell = min(temp2)
		drop temp*
		
		***
		*** Define spell length (how long since last new price)
		***
		xtset group_id WEEK
		* Start a spell either with a new price or start of quote line (**IMP**: can be adjusted for censoring now or later)
		gen last_change_any=WEEK if anyprice_ok==1|spell_start==1
		replace last_change_any=L.last_change_any if anyprice_ok==0&last_change_any==.
		gen spell_any = WEEK-last_change_any+1
		gen lag_spell_any = L.spell_any
	
		*** Price change dummy
		gen dp = ln(P/L.P)
		gen temp1 = abs(dp)
		gen dpdum = 0 if dp!=.
		replace dpdum = 1 if dp!=.&dp!=0&temp1>=0.01
		drop temp*				
				
		* Define market size (for weighted regression)
		egen sum_DOLLARS = total(DOLLARS)
		bysort group_id: egen sum_DOLLARS_group_id = total(DOLLARS)


		*********************************
		* ITEM and IRI_KEY fixed effects
		*********************************
		drop if lag_spell_any==.
		drop if lag_spell_any>26
	
		* Drop all left-censored spells
		reghdfe dpdum lag_spell_any if lag_spell_any<=26&left_censored!=1, absorb(ITEM IRI_KEY WEEK) cluster(IRI_KEY)
		gen coeff_spell_slope= _b[lag_spell_any]
		gen se_spell_slope= _se[lag_spell_any]
		gen r2 = e(r2)
		gen nobs = e(N)

		
		* Drop all left-censored spells (group_ids are dollar weighted)
		reghdfe dpdum lag_spell_any [aw=sum_DOLLARS_group_id] if lag_spell_any<=26&left_censored!=1, absorb(ITEM IRI_KEY WEEK) cluster(IRI_KEY)
		gen weighted_coeff_spell_slope= _b[lag_spell_any]
		gen weighted_se_spell_slope= _se[lag_spell_any]
		gen weighted_r2 = e(r2)
		gen weighted_nobs = e(N)


		**
		** COLLAPSE DATA
		**

		collapse (mean) *coeff_spell_slope* *se_spell_slope* *r2 *nobs (sum) DOLLARS, by(CAT MKT)


		*** SAVE DATA (at ITEM level)
		append using "data/dbase_for_hazard_slope_april2020"
		save "data/dbase_for_hazard_slope_april2020", replace
		
		
	}		

}




***
*** Histograms (Appendix B.5)
***

clear all
use "./data/dbase_for_hazard_slope_april2020"
*use "I:\data\IRI\fear\hazards_reg_by_item_forpaper_26w_15jan2019.dta"


hist coeff_spell_slope, title("Unweighted") xtitle("Slope")   graphregion(fcolor(white) ilcolor(white) lcolor(white)) name(g1, replace)
*hist v5_coeff_spell_slope_late, title("Slope for {&tau}{&ge}4 price spells") subtitle("Unweighted") xtitle("Slope") name(g2, replace)
hist weighted_coeff_spell_slope, title("Weighted") xtitle("Slope")   graphregion(fcolor(white) ilcolor(white) lcolor(white)) name(g3, replace)
*hist v7_coeff_spell_slope_late, title("Slope for {&tau}{&ge}4 price spells") subtitle("Weighted") xtitle("Slope") name(g4, repl
graph combine g1 g3,  graphregion(fcolor(white) ilcolor(white) lcolor(white))

/*
gen v4_early_tstat = v4_coeff_spell_slope_early/v4_se_spell_slope_early
gen v4_late_tstat = v4_coeff_spell_slope_late/v4_se_spell_slope_late
gen v9_early_tstat = v9_coeff_spell_slope_early/v9_se_spell_slope_early
gen v9_late_tstat = v9_coeff_spell_slope_late/v9_se_spell_slope_late
*/

*merge 1:1 MKT CAT using "I:/data/IRI/fear/weights_for_moments_MKT_CAT_may2017"
*drop _merge


*sum v4_coeff_spell_slope v5_coeff_spell_slope_late v6_coeff_spell_slope v7_coeff_spell_slope_late
*sum v4_coeff_spell_slope v5_coeff_spell_slope_late v6_coeff_spell_slope v7_coeff_spell_slope_late [aw=DOLLARS]

collapse (mean) *coeff_spell_slope  [aw=DOLLARS]

**** TABLE III, panel C + text ****
quietly {
noisily: display as text "Avg hazard slope = " coeff_spell_slope
noisily: display as text "Avg hazard slope (weighted regression) = " weighted_coeff_spell_slope
}
