********************************************************************************************************************************************
** This code does two main things:
**	Part 1: For each category/market, generates a number of moments
**  Part 2: Creates aggregate moments (weighted across cat/mkt)
********************************************************************************************************************************************


clear
clear matrix
clear mata
set more off
set matsize 600
set type double


** <<IMPORTANT>> Note: Use directory of this code as working directory


********
******** Part 1: For each category/market, generates a number of moments
********

save "data/dbase_for_main_moments_april2020", emptyok replace


foreach MKT in ATL BOS CHI DAL HOU LA NYC PHI SF {
	foreach CAT in beer coldcer fzdinent fzpizza saltsnck yogurt{

		***
		*** Load data
		***

		local infilename ="data/allprices_allyears_wsales_"+"`CAT'"+"_"+"`MKT'"
		use "`infilename'"
		
		gen CAT = "`CAT'"
		gen MKT = "`MKT'"
		display "`MKT'"
		display "`CAT'"
		
		xtset group_id WEEK

		
		*** Define relevant price variable (regular price)
		gen p = REGPRICE
		drop if p==.

		
		*** Keep only observations for which the price is to the cent (to limit spurious price changes)
		gen p_dec_100_cent_round = round((p-floor(p))*100-round((p-floor(p))*100),0.001)
		drop if p_dec_100_cent_round~=0
		drop p_*
	
				
		*** Price changes 
		gen dlp=ln(p)-ln(L.p)
		gen dlp_abs = abs(dlp)	
		gen dlp_nonzero=dlp
		replace dlp_nonzero=. if dlp_nonzero==0
		gen dlp_abs_nonzero = dlp_abs
		replace dlp_abs_nonzero = . if dlp_abs==0
		

		*** Price change indicators
		gen dp_up = (dlp>0&dlp!=.)
		gen dp_chg = (dlp_abs>0&dlp_abs!=.)
		foreach var in dp_chg dp_up {
			replace `var' = . if dlp==.
		}


		******
		****** Kurtosis (drop bottom 0.005 and top 0.995)
		******
		gquantiles pctile_dum = dlp_nonzero, xtile nquantiles(200)
		sum dlp_nonzero if pctile_dum>1&pctile_dum<199, detail
		gen kurt=r(kurtosis)
		
		
		******
		****** Modal price and unique prices in each window
		******
		
		
		*** Week identifier
		gen w1=ceil((WEEK-1000+1)/13)
		gen w2=ceil((WEEK-1001+1)/13)
		gen w3=ceil((WEEK-1002+1)/13)
		gen w4=ceil((WEEK-1003+1)/13)
		gen w5=ceil((WEEK-1004+1)/13)
		gen w6=ceil((WEEK-1005+1)/13)
		gen w7=ceil((WEEK-1006+1)/13)
		gen w8=ceil((WEEK-1007+1)/13)
		gen w9=ceil((WEEK-1008+1)/13)
		gen w10=ceil((WEEK-1009+1)/13)
		gen w11=ceil((WEEK-1010+1)/13)
		gen w12=ceil((WEEK-1011+1)/13)
		gen w13=ceil((WEEK-1012+1)/13)
		
		
		
		*** Modal price and unique prices in each window		
		forvalues x=1/13{
			bysort group_id w`x': egen nbobs_w`x'=count(p)
			egen temp_w`x'=group(group_id w`x' p)
			bysort group_id w`x':egen temp2_w`x'=min(temp_w`x')
			bysort group_id w`x':egen temp3_w`x'=max(temp_w`x')
			* Min acceptable nb of observations within window (unique # of prices)
			bysort group_id w`x':egen modalp_w`x'=mode(p),maxmode
			* Min acceptable nb of observations within window (modal price)
			replace modalp_w`x'=. if nbobs_w`x'<4
		}	
		gen modalp=modalp_w1
		forvalues x=2/13{
			gen temp44 = 1007+(`x'-2)
			gen z=(WEEK-temp44)/13-round((WEEK-temp44)/13)
			replace modalp=modalp_w`x' if z==0
			drop z temp44			
		}
				
	
		
		
		***
		*** DEFINE LAGS AND LEADS OF PRICE
		***
		
		xtset group_id WEEK
		
		forvalues x = 1/26{
			gen l`x'p = L`x'.p
		}
		
		forvalues x = 1/6{
			gen f`x'p = F`x'.p
		}
		
		
		************
		************ MODE/REFPRICE MOMENTS
		************
		
		*** Use modal price as the reference price
		drop REFPRICE
		gen REFPRICE = modalp

		*** Nb of observations within window
		egen nbobs=rownonmiss(l1p l2p l3p l4p l5p l6p p f1p f2p f3p f4p f5p f6p)	
		
		*** Fraction of time at modal price  -- MINIMUM of 8 observations
		gen nb_at_modal = (l1p==REFPRICE)+(l2p==REFPRICE)+(l3p==REFPRICE)+(l4p==REFPRICE)+(l5p==REFPRICE)+(l6p==REFPRICE)+(p==REFPRICE)+(f1p==REFPRICE)+(f2p==REFPRICE)+(f3p==REFPRICE)+(f4p==REFPRICE)+(f5p==REFPRICE)+(f6p==REFPRICE)
		gen frac_at_modal = nb_at_modal/nbobs
		replace frac_at_modal=. if REFPRICE==.|nbobs<8
		
		*** Max vs reference price  -- MINIMUM of 8 observations
		egen maxp=rowmax(l1p l2p l3p l4p l5p l6p p f1p f2p f3p f4p f5p f6p)
		gen max_is_modal=0 if maxp!=.&REFPRICE!=.
		replace max_is_modal=1 if maxp==REFPRICE
		replace max_is_modal=. if REFPRICE==.|nbobs<8
		
	
		*** Probability of a price change ending at reference/modal price CONDITIONAL
		* on the price it comes from not having been a modal price over the last 13 weeks
		gen dp_chg_to_modal = 0 if dp_chg==1
		replace dp_chg_to_modal = 1 if dp_chg==1&p==REFPRICE
		replace dp_chg_to_modal =. if L.p==L.REFPRICE|L.p==L2.REFPRICE|L.p==L3.REFPRICE|L.p==L4.REFPRICE|L.p==L5.REFPRICE|L.p==L6.REFPRICE|L.p==L7.REFPRICE|L.p==L8.REFPRICE|L.p==L9.REFPRICE|L.p==L10.REFPRICE|L.p==L11.REFPRICE|L.p==L12.REFPRICE|L.p==L13.REFPRICE

		
		************
		************ MEMORY MOMENTS
		************
		
		gen P = p
		* Identify whether a price was posted before (26-weeks window)
		egen nlags = rownonmiss(l1p l2p l3p l4p l5p l6p l7p l8p l9p l10p l11p l12p l13p l14p l15p l16p l17p l18p l19p l20p l21p l22p l23p l24p l25p l26p)
		gen oldprice6 = 0 if dp_chg==1&P!=.&L.P!=.&nlags>=13
		replace oldprice6 = 1 if dp_chg==1 & nlags>=13 & (P==L.P|P==L2.P|P==L3.P|P==L4.P|P==L5.P|P==L6.P|P==L7.P|P==L8.P|P==L9.P|P==L10.P|P==L11.P|P==L12.P|P==L13.P|P==L14.P|P==L15.P|P==L16.P|P==L17.P|P==L18.P|P==L19.P|P==L20.P|P==L21.P|P==L22.P|P==L23.P|P==L24.P|P==L25.P|P==L26.P)

		* Fix if there has been no other price change over past 26 weeks (so cannot go back to old price by definition)
		replace oldprice6 = . if dp_chg==1 & (L.dp_chg==0&L2.dp_chg==0&L3.dp_chg==0&L4.dp_chg==0&L5.dp_chg==0&L6.dp_chg==0&L7.dp_chg==0&L8.dp_chg==0&L9.dp_chg==0&L10.dp_chg==0&L11.dp_chg==0&L12.dp_chg==0&L13.dp_chg==0&L14.dp_chg==0&L15.dp_chg==0&L16.dp_chg==0&L17.dp_chg==0&L18.dp_chg==0&L19.dp_chg==0&L20.dp_chg==0&L21.dp_chg==0&L22.dp_chg==0&L23.dp_chg==0&L24.dp_chg==0&L25.dp_chg==0&L26.dp_chg==0)

		* Fix if not enough price observations over last 26 weeks (minimum of 13 is required)
		replace oldprice6 = . if nlags<13

		*** Reference (modal) price change indicators
		gen dlprefp = ln(REFPRICE/L.REFPRICE)
		gen drefp_up = (dlprefp>0&dlprefp!=.)
		gen drefp_dn = (dlprefp<0&dlprefp!=.)
		gen drefp_chg = (dlprefp!=0&dlprefp!=.)
		foreach var in drefp_chg drefp_dn drefp_chg{
			replace `var' = . if dlprefp==.
		}

		
		***********
		* Collapse across products (within a specific category/market)
		***********
		collapse (p50) dlp_abs_p50=dlp_abs_nonzero (p75) dlp_abs_p75=dlp_abs_nonzero (mean) dp_chg dp_up drefp_chg  frac_at_modal  max_is_modal  dp_chg_to_modal  oldprice6 kurt (sum) DOLLARS , by(CAT MKT)
		
		append using "data/dbase_for_main_moments_april2020"
		save "data/dbase_for_main_moments_april2020", replace
		
		

	}		

}



******************************************************************************************************
*** WEIGHTED MOMENTS (across category & markets)
******************************************************************************************************
		
collapse (mean) dlp_abs_p50 dlp_abs_p75 dp_chg dp_up drefp_chg  frac_at_modal  max_is_modal  dp_chg_to_modal  oldprice6 kurt   [aw=DOLLARS]


******************************************************************************************************
**************** MOMENTS :  Value and description
******************************************************************************************************


**** TABLE II (Section 4.2) ****
quietly {
noisily: display as text "Frequency of regular price changes = " dp_chg
noisily: display as text "Median size of absolute regular price changes = " dlp_abs_p50
noisily: display as text "75th pctile of the distribution of non-zero absolute price changes = " dlp_abs_p75
noisily: display as text "Fraction of non-zero price changes that are increases = " dp_up/dp_chg
noisily: display as text "Frequency of modal price changes (13-week window) = " drefp_chg
}


**** TABLE III, panel A (Section 4.2.1) ****
quietly {
noisily: display as text "Prob. modal P is max P = " max_is_modal
noisily: display as text "Fraction of weeks at modal P (13-week window) = " frac_at_modal
noisily: display as text "Prob. price moves to modal P = " dp_chg_to_modal
}

**** TABLE III, panel B (Section 4.2.1) ****
quietly {
noisily: display as text "Prob. revisiting old price (26-week window) = " oldprice6
}


**** TABLE III, panel D (Section 4.2.1) ****
quietly {
noisily: display as text "Kurtosis (26-week window) = " kurt
}
