********************************************************************************************************************************************
** This code generates the z-score regression results of Table 4, Section 4.2.4
**  Preliminary step: create basic database with z-scores and other variables by running create_dbase_zscores_april2020.do
**
**	Part 1: Creates the database
**  Part 2: Run regressions
********************************************************************************************************************************************

cls
clear
clear matrix
clear mata
set more off
set matsize 6000

** <<IMPORTANT>> Note: Use directory of this code as working directory


***
*** PART 1: Create database for regression variables
***

* Loads the database previously created by "create_dbase_zscores_april2020.do"
save "./data/dbase_for_zscore_regressions_april2020", emptyok replace

foreach MKT in ATL BOS CHI DAL HOU LA NYC PHI SF {
	foreach CAT in beer coldcer fzdinent fzpizza saltsnck yogurt{

		clear
		
		***
		*** Load data
		***

		local infilename ="data/dbase_zscores_april2020_"+"`CAT'"+"_"+"`MKT'"
		use "`infilename'"
		
		
		
		disp CAT
		disp MKT
		xtset group_id WEEK

		*** Keep only observations for which the price is to the cent
		gen p=REGPRICE
		gen p_dec_100_cent_round = round((p-floor(p))*100-round((p-floor(p))*100),0.001)
		drop if p_dec_100_cent_round~=0
		drop p_*
		
		
		*gen d_PRICE_ORIG=ln(PRICE_ORIG/L.PRICE_ORIG)
		gen d_REGPRICE=ln(REGPRICE/L.REGPRICE)
		quietly: gen dpregdum = 0 if  d_REGPRICE==0
		quietly: replace dpregdum=1 if d_REGPRICE!=0&d_REGPRICE!=.
		

		
		*****
		***** Variables and indicators for regressions
		***** 
		
		gen diff = L.zscore26_2_avg - L.zscore26_2_laginfo_avg
		gen x1=diff
		gen x2=L.wscore26
		
		*** Only keep valid observations (including enough lags for score computation)
		gen validobs_13 = 1 if dpregdum!=.&L.dpregdum==0&x1!=.&x2!=.&nblags_wscore>=20&x2<=12
		gen validobs_26 = 1 if dpregdum!=.&L.dpregdum==0&x1!=.&x2!=.&nblags_wscore>=20&x2<=25

		drop if validobs_13!=1&validobs_26!=1
		
				
		keep CAT MKT WEEK ITEM group_id dpregdum x1 x2 valid* DOLLARS
		
		compress 
		
		***
		*** Collapse and append 
		***
				
		append using "data/dbase_for_zscore_regressions_april2020"
		save "data/dbase_for_zscore_regressions_april2020", replace

		
	}		

}



**********************
*** REGRESSIONS
**********************


rename group_id group_id_old
egen group_id=group(MKT CAT group_id_old)
egen group_cat_mkt=group(MKT CAT)

xtset group_id WEEK
		

*** Other variables (to obtain normalized impacts)

quietly: gen x1v=x1 if validobs_13==1
quietly: egen sd_x1score_1=sd(x1v)
quietly: gen x2v=x2 if validobs_13==1
quietly: egen sd_x2score_1=sd(x2v)		
quietly: gen y=dpregdum if validobs_13==1
quietly: egen sd_y_1=sd(y)
quietly: egen avg_y_1=mean(y)

drop x1v x2v y

quietly: gen x1v=x1 if validobs_26==1
quietly: egen sd_x1score_2=sd(x1v)
quietly: gen x2v=x2 if validobs_26==1
quietly: egen sd_x2score_2=sd(x2v)		
quietly: gen y=dpregdum if validobs_26==1
quietly: egen sd_y_2=sd(y)
quietly: egen avg_y_2=mean(y)

drop x1v x2v y


compress


*********** REGRESSIONS - 13 weeks *********

**** Cat/Mkt fixed effects
reghdfe dpregdum x1 x2 if validobs_13==1, absorb(group_cat_mkt)  cluster(group_cat_mkt)
quietly: gen x1reg_13w_fe1_coeff = _b[x1]
quietly:gen x1reg_13w_fe1_se = _se[x1]
quietly: gen x2reg_13w_fe1_coeff = _b[x2]
quietly:gen x2reg_13w_fe1_se = _se[x2]
quietly:gen reg_13w_fe1_nobs = e(N)
quietly:gen reg_13w_fe1_r2 = e(r2)	

**** Group_id fixed effects
reghdfe dpregdum x1 x2 if validobs_13==1, absorb(group_id)  cluster(group_cat_mkt)
quietly: gen x1reg_13w_fe2_coeff = _b[x1]
quietly:gen x1reg_13w_fe2_se = _se[x1]
quietly: gen x2reg_13w_fe2_coeff = _b[x2]
quietly:gen x2reg_13w_fe2_se = _se[x2]
quietly:gen reg_13w_fe2_nobs = e(N)
quietly:gen reg_13w_fe2_r2 = e(r2)	



*********** REGRESSIONS - 26 weeks *********

**** Cat/Mkt fixed effects
reghdfe dpregdum x1 x2 if validobs_26==1, absorb(group_cat_mkt)  cluster(group_cat_mkt)
quietly: gen x1reg_26w_fe1_coeff = _b[x1]
quietly:gen x1reg_26w_fe1_se = _se[x1]
quietly: gen x2reg2_26w_fe1_coeff = _b[x2]
quietly:gen x2reg2_26w_fe1_se = _se[x2]
quietly:gen reg2_26w_fe1_nobs = e(N)
quietly:gen reg2_26w_fe1_r2 = e(r2)	

**** Group_id fixed effects
reghdfe dpregdum x1 x2 if validobs_26==1, absorb(group_id)  cluster(group_cat_mkt)
quietly: gen x1reg_26w_fe2_coeff = _b[x1]
quietly:gen x1reg_26w_fe2_se = _se[x1]
quietly: gen x2reg_26w_fe2_coeff = _b[x2]
quietly:gen x2reg_26w_fe2_se = _se[x2]
quietly:gen reg_26w_fe2_nobs = e(N)
quietly:gen reg_26w_fe2_r2 = e(r2)	

		

*** Collapse		
keep if _n==1
collapse (first) sd_x1score_1 sd_x2score_1 sd_x1score_2 sd_x2score_2 sd_y_1 avg_y_1 sd_y_2 avg_y_2 x1reg* x2reg* reg*


*** t-stat
gen z_tstat_13w_fe1=abs( x1reg_13w_fe1_coeff/ x1reg_13w_fe1_se)
gen z_tstat_13w_fe2=abs( x1reg_13w_fe2_coeff/ x1reg_13w_fe2_se)
gen z_tstat_26w_fe1=abs( x1reg_26w_fe1_coeff/ x1reg_26w_fe1_se)
gen z_tstat_26w_fe2=abs( x1reg_26w_fe2_coeff/ x1reg_26w_fe2_se)

** Impact measures (z)
gen x1_impact_reg_13w_fe1 = sd_x1score_1*x1reg_13w_fe1_coeff
gen x1_impact_reg_13w_fe2 = sd_x1score_1*x1reg_13w_fe2_coeff
gen x1_impact_reg_26w_fe1 = sd_x1score_2*x1reg_26w_fe1_coeff
gen x1_impact_reg_26w_fe2 = sd_x1score_2*x1reg_26w_fe2_coeff

** Impact measures (w)
gen x2_impact_reg_13w_fe1 = sd_x2score_1*x2reg_13w_fe1_coeff
gen x2_impact_reg_13w_fe2 = sd_x2score_1*x2reg_13w_fe2_coeff
gen x2_impact_reg_26w_fe1 = sd_x2score_2*x2reg_26w_fe1_coeff
gen x2_impact_reg_26w_fe2 = sd_x2score_2*x2reg_26w_fe2_coeff


*************************************************
**** Table IV, columns 1 to 4, Section 4.2.4 ****
*************************************************
quietly {
noisily: display as text "Marginal effect of z-score change on price chg probability, 13 weeks, CAT/MKT FE = " x1_impact_reg_13w_fe1
noisily: display as text "Marginal effect of z-score change on price chg probability, 13 weeks, prod/store FE = " x1_impact_reg_13w_fe2
noisily: display as text "Marginal effect of z-score change on price chg probability, 26 weeks, CAT/MKT FE = " x1_impact_reg_26w_fe1
noisily: display as text "Marginal effect of z-score change on price chg probability, 26 weeks, prod/store FE = " x1_impact_reg_26w_fe2
}
