
function [brks,SnVals] = break_test_basic(series,varargin) 

% -------------------------------------------------------------------------  
%  break_test_basic.m
%
%  Project: “Coarse Pricing Policies?
%           Luminita Stevens, University of Maryland
%
%  Description: Implements break test for a given series. Adaptation of  
%   Kolmogorov-Smirnov two-sample test to multiple potential breaks at 
%   unknown locations. 
%
%  Input: 
%   series  :  vector of prices 
%   window  :  minimum possible regime length (default=1) 
%   crit    :  critical value for rejecting null of no break (default=0.61)
%             (default value is based on simulations)
%   
%  Output: 
%   brks    : vector of 0s and 1s with 1s at index of breaks  
%   SnVals  : vector of test statistics 
%
%  Calls getbreak.m
% -------------------------------------------------------------------------

if (nargin < 2), window = 1;  else window = varargin{1}; end
if (nargin < 3), crit = 0.61; else crit = varargin{2}; end

% Initializations. 
ns      = length(series); 
brks    = zeros(ns,1);
SnVals  = 0;

% Done if not enough observations relative to the minimum regime length.
if ns<2*window, return, end

% Preallocations. 
max_regs = floor(ns./window);                % max possible # regimes 
max_iter = max_regs+1;                       % max possible # iterations 
mn_iter  = zeros(max_regs,max_iter);         % begining-of-regime indexing
ns_iter  = zeros(max_regs,max_iter);         % end-of-regime indexing
tau_iter = zeros(max_regs-1,max_regs-1);     % break indexing 

% First iteration, on full sample.
mn_iter(1,1) = 1; 
ns_iter(1,1) = ns; 
srs = series(mn_iter(1,1):ns_iter(1,1)); 
[t_loc,s_val] = getbreak(srs,crit);
tau_iter(1,1) = mn_iter(1,1) + t_loc - 1 ;   % start of next regime
   
% Done if found no break or found a break that violates min regime length. 
if tau_iter(1,1) < 0 || tau_iter(1,1)-mn_iter(1,1) < window || ns_iter(1,1)-tau_iter(1,1)+1 < window, return, end 

% Otherwise save position of break, and value and position of statistic.
brks(tau_iter(1,1)) = 1;         
SnVals = s_val;  

% Iterating on the resulting subsamples.
for it=2:max_regs-1
    
    % Branches with breaks    
    breaks = find(tau_iter(it-1,:)>0);
    
    for bp = 1:length(breaks)
        
        sp = breaks(bp);
        
        % -----------------------------------------------------------------
        % odd sample inherits parent's start point, parent's break-1 
        % -----------------------------------------------------------------
        mn_iter(it,2*sp-1) = mn_iter(it-1,sp); 
        ns_iter(it,2*sp-1) = tau_iter(it-1,sp) - 1;
        
        % confirm length of series >=2*window
        if ns_iter(it,2*sp-1) - mn_iter(it,2*sp-1) + 1 >= 2*window        

            % get breakpoint if it exists
            srs = series(mn_iter(it,2*sp-1): ns_iter(it,2*sp-1)); 
            [t_loc,s_val] = getbreak(srs, crit); 
            tau_iter(it,2*sp-1) = mn_iter(it,2*sp-1) + t_loc - 1;   % start of next regime
            
            % if no break or resulting subsamples are too short, reset to 0
            if tau_iter(it,2*sp-1) < 0 || tau_iter(it,2*sp-1)-mn_iter(it,2*sp-1) < window ...
                    || ns_iter(it,2*sp-1)-tau_iter(it,2*sp-1)+1 < window
                
                tau_iter(it,2*sp-1) = 0;
                
            % otherwise record position
            else
                
                brks(tau_iter(it,2*sp-1)) = 1;
                SnVals = [SnVals; s_val];
                
                
            end
            
        end
        
        % -----------------------------------------------------------------
        % even sample inherits parent's break, parent's end point
        % -----------------------------------------------------------------
        mn_iter(it,2*sp) = tau_iter(it-1,sp);
        ns_iter(it,2*sp) = ns_iter(it-1,sp);
        
        % confirm length of series >=2*window
        if ns_iter(it,2*sp) - mn_iter(it,2*sp) + 1 >= 2*window        

            % get breakpoint if it exists
            srs = series(mn_iter(it,2*sp): ns_iter(it,2*sp)); 
            [t_loc,s_val] = getbreak(srs, crit); 
            tau_iter(it,2*sp) = mn_iter(it,2*sp) + t_loc - 1;   % start of next regime
            
            % if no break or resulting subsamples are too short, reset to 0
            if tau_iter(it,2*sp) < 0 || tau_iter(it,2*sp)-mn_iter(it,2*sp) < window ...
                    || ns_iter(it,2*sp)-tau_iter(it,2*sp)+1 < window
                
                tau_iter(it,2*sp) = 0;
                
            % otherwise record position
            else
                
                brks(tau_iter(it,2*sp)) = 1;
                SnVals = [SnVals; s_val];
                
                
            end
            
        end
        
    end    
    
    
    % Done if found no break
    if sum(tau_iter(it,:))==0, return, end 
    
end


end 


