
function [ksIndex,SnVal] = getbreak(series,crit)
    
% -------------------------------------------------------------------------  
%  getbreak.m
%
%  Project: “Coarse Pricing Policies”
%           Luminita Stevens, University of Maryland
%
%  Description: Test for a single break on a given sample
%
%  Input: 
%   series  :  vector of observations 
%   crit    :  critical value for rejecting null of no break 
%
%  Output:    (-1.e6 if fail to reject null)
%   ksIndex : index of Carlstein statistic tau_n   
%   SnVal   : value of Deshayes-Picard test statistic  
%
%  Calls getdiff.m
% -------------------------------------------------------------------------

% Initializations. 
ksIndex  = -1.e6;  
SnVal    = -1.e6;  

% Remove missing observations. 
series  =  series(~isnan(series));

% For each potential break point t in the series compute Dn(t) and norm(t), 
% the K-S two-sample distance and normalization factor. 
N    = size(series,1);  
S1   = triu(series*ones(1,N-1)) + tril(nan(N,N-1),-1);
S2   = tril(series*ones(1,N-1)) + triu(nan(N,N-1));

[Dn, norm] = getdiff(S1,S2); 

% Compute Deshayes-Picard and Carlstein statistics.  
Dnt   =  norm.*Dn;
DP    =  sqrt(N)*max(Dnt) ;
Taut  =  sqrt(norm.*N).*Dn ;
CA    =  max(Taut); 

% Update return variables, if null is rejected. 
if DP >= crit    % reject null 
    
    SnVal = DP; 
    % SnIndex=1+find(Dnt == max(Dnt),1,'first'); % +1 bc Dnt starts @ index 2 

    ksIndex=1+find(Taut == CA, 1,'first');    % +1 bc Taut starts @ index 2
    
end

end 